@@ -5,6 +5,7 @@ import pandas as pd
 import numpy as np
 import numpy.linalg as la
 import math
+import copy
 
 def neuroCombat(dat,
            covars,
@@ -21,19 +22,19 @@ def neuroCombat(dat,
     Arguments
     ---------
     dat : a pandas data frame or numpy array
-        neuroimaging data to correct with shape = (features, samples)
-        e.g. cortical thickness measurements, image voxels, etc
+        - neuroimaging data to correct with shape = (features, samples) e.g. cortical thickness measurements, image voxels, etc
 
     covars : a pandas data frame w/ shape = (samples, covariates)
-        demographic/phenotypic/behavioral/batch data 
+        - contains the batch/scanner covariate as well as additional covariates (optional) that should be preserved during harmonization.
         
-    batch_col : string indicating batch (scanner) column name in covars
+    batch_col : string
+        - indicates batch (scanner) column name in covars (e.g. "scanner")
 
-    categorical_cols : string or list of strings of categorical variables to adjust for
-        - e.g. male or female
+    categorical_cols : list of strings
+        - specifies column names in covars data frame of categorical variables to be preserved during harmonization (e.g. ["sex", "disease"])
 
-    continuous_cols : string or list of strings of continuous variables to adjust for
-        - e.g. age
+    continuous_cols : list of strings
+        - indicates column names in covars data frame of continuous variables to be preserved during harmonization (e.g. ["age"])
 
     eb : should Empirical Bayes be performed?
         - True by default
@@ -49,7 +50,10 @@ def neuroCombat(dat,
         
     Returns
     -------
-    - A numpy array with the same shape as `dat` which has now been ComBat-harmonized
+    A dictionary of length 3:
+    - data: A numpy array with the same shape as `dat` which has now been ComBat-harmonized
+    - estimates: A dictionary of the ComBat estimates used for harmonization
+    - info: A dictionary of the inputs needed for ComBat harmonization
     """
     ##############################
     ### CLEANING UP INPUT DATA ###
@@ -70,11 +74,6 @@ def neuroCombat(dat,
 
     covar_labels = np.array(covars.columns)
     covars = np.array(covars, dtype='object') 
-    for i in range(covars.shape[-1]):
-        try:
-            covars[:,i] = covars[:,i].astype('float32')
-        except:
-            pass
 
     if isinstance(dat, pd.DataFrame):
         dat = np.array(dat, dtype='float32')
@@ -103,22 +102,29 @@ def neuroCombat(dat,
             ref_level = covars[np.int(ref_indices[0]),batch_col]
     # create dictionary that stores batch info
     (batch_levels, sample_per_batch) = np.unique(covars[:,batch_col],return_counts=True)
+    print(batch_levels)
+
+     # create design matrix
+    print('[neuroCombat] Creating design matrix')
+    design = make_design_matrix(covars, batch_col, cat_cols, num_cols, ref_level)
+    
+
     info_dict = {
         'batch_levels': batch_levels,
         'ref_level': ref_level,
         'n_batch': len(batch_levels),
         'n_sample': int(covars.shape[0]),
         'sample_per_batch': sample_per_batch.astype('int'),
-        'batch_info': [list(np.where(covars[:,batch_col]==idx)[0]) for idx in batch_levels]
+        'batch_info': [list(np.where(covars[:,batch_col]==idx)[0]) for idx in batch_levels],
+        'design': design
     }
 
-    # create design matrix
-    print('[neuroCombat] Creating design matrix')
-    design = make_design_matrix(covars, batch_col, cat_cols, num_cols, ref_level)
     
+
+   
     # standardize data across features
     print('[neuroCombat] Standardizing data across features')
-    s_data, s_mean, v_pool = standardize_across_features(dat, design, info_dict)
+    s_data, s_mean, v_pool, mod_mean = standardize_across_features(dat, design, info_dict)
     
     # fit L/S models and find priors
     print('[neuroCombat] Fitting L/S model and finding priors')
@@ -142,8 +148,14 @@ def neuroCombat(dat,
                                     s_mean, v_pool, info_dict,dat)
 
     bayes_data = np.array(bayes_data)
+    estimates = {'batches': info_dict['batch_levels'], 'var.pooled': v_pool, 'stand.mean': s_mean, 'mod.mean': mod_mean, 'gamma.star': gamma_star, 'delta.star': delta_star}
+    estimates = {**LS_dict, **estimates, }
 
-    return bayes_data
+    return {
+        'data': bayes_data,
+        'estimates': estimates,
+        'info': info_dict
+    }
 
 
 
@@ -209,15 +221,13 @@ def standardize_across_features(X, design, info_dict):
     B_hat = np.vstack(betas).T
     
     #B_hat = np.dot(np.dot(la.inv(np.dot(design.T, design)), design.T), X.T)
-    if ref_level is None:
-        grand_mean = np.dot((sample_per_batch/ float(n_sample)).T, B_hat[:n_batch,:])
-    else:
+    if ref_level is not None:
         grand_mean = np.transpose(B_hat[ref_level,:])
+    else:
+        grand_mean = np.dot((sample_per_batch/ float(n_sample)).T, B_hat[:n_batch,:])
     stand_mean = np.dot(grand_mean.T.reshape((len(grand_mean), 1)), np.ones((1, n_sample)))
     #var_pooled = np.dot(((X - np.dot(design, B_hat).T)**2), np.ones((n_sample, 1)) / float(n_sample))
 
-    ######### Continue here. 
-
     if ref_level is not None:
         X_ref = X[:,batch_info[ref_level]]
         design_ref = design[batch_info[ref_level],:]
@@ -227,14 +237,22 @@ def standardize_across_features(X, design, info_dict):
         var_pooled = np.dot(((X - np.dot(design, B_hat).T)**2), np.ones((n_sample, 1)) / float(n_sample))
 
     var_pooled[var_pooled==0] = np.median(var_pooled!=0)
+    
+    mod_mean = 0
+    if design is not None:
+        tmp = copy.deepcopy(design)
+        tmp[:,range(0,n_batch)] = 0
+        mod_mean = np.transpose(np.dot(tmp, B_hat))
+    ######### Continue here. 
 
-    tmp = np.array(design.copy())
-    tmp[:,:n_batch] = 0
-    stand_mean  += np.dot(tmp, B_hat).T
 
-    s_data = ((X- stand_mean) / np.dot(np.sqrt(var_pooled), np.ones((1, n_sample))))
+    #tmp = np.array(design.copy())
+    #tmp[:,:n_batch] = 0
+    #stand_mean  += np.dot(tmp, B_hat).T
 
-    return s_data, stand_mean, var_pooled
+    s_data = ((X- stand_mean - mod_mean) / np.dot(np.sqrt(var_pooled), np.ones((1, n_sample))))
+
+    return s_data, stand_mean, var_pooled, mod_mean
 
 def aprior(delta_hat):
     m = np.mean(delta_hat)
@@ -424,3 +442,68 @@ def adjust_data_final(s_data, design, gamma_star, delta_star, stand_mean, var_po
         bayesdata[:, batch_info[ref_level]] = dat[:,batch_info[ref_level]]
 
     return bayesdata
+
+
+
+
+def neuroCombatFromTraining(dat,
+                            batch,
+                            estimates):
+    """
+    Combat harmonization with pre-trained ComBat estimates [UNDER DEVELOPMENT]
+
+    Arguments
+    ---------
+    dat : a pandas data frame or numpy array for the new dataset to harmonize
+        - rows must be identical to the training dataset
+    
+    batch : numpy array specifying scanner/batch for the new dataset
+        - scanners/batches must also be present in the training dataset
+
+    estimates : dictionary of ComBat estimates from a previously-harmonized dataset
+        - should be in the same format as neuroCombat(...)['estimates']
+        
+    Returns
+    -------
+    A dictionary of length 2:
+    - data: A numpy array with the same shape as `dat` which has now been ComBat-harmonized
+    - estimates: A dictionary of the ComBat estimates used for harmonization
+    """
+    print("[neuroCombatFromTraining] In development ...\n")
+    batch = np.array(batch, dtype="str")
+    new_levels = np.unique(batch)
+    old_levels = np.array(estimates['batches'], dtype="str")
+    missing_levels = np.setdiff1d(new_levels, old_levels)
+    if missing_levels.shape[0] != 0:
+        raise ValueError("The batches " + str(missing_levels) +
+                         " are not part of the training dataset")
+
+
+    wh = [int(np.where(old_levels==x)[0]) if x in old_levels else None for x in batch]
+
+    
+
+    var_pooled = estimates['var.pooled']
+    stand_mean = estimates['stand.mean'][:, 0]
+    mod_mean = estimates['mod.mean']
+    gamma_star = estimates['gamma.star']
+    delta_star = estimates['delta.star']
+    n_array = dat.shape[1]   
+    stand_mean = stand_mean+mod_mean.mean(axis=1)
+ 
+    
+    stand_mean = np.transpose([stand_mean, ]*n_array)
+    bayesdata = np.subtract(dat, stand_mean)/np.sqrt(var_pooled)
+    
+    #gamma = np.transpose(np.repeat(gamma_star, repeats=2, axis=0))
+    #delta = np.transpose(np.repeat(delta_star, repeats=2, axis=0))
+    gamma = np.transpose(gamma_star[wh,:])
+    delta = np.transpose(delta_star[wh,:])
+    bayesdata = np.subtract(bayesdata, gamma)/np.sqrt(delta)
+    
+    bayesdata = bayesdata*np.sqrt(var_pooled) + stand_mean
+    out = {
+        'data': bayesdata,
+        'estimates': estimates
+    }
+    return out
